Type: Package
Package: r5r
Title: Rapid Realistic Routing with 'R5'
Version: 2.4.0
Authors@R: c(
    person("Marcus", "Saraiva", , "marcus.saraiva@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0001-6218-2338")),
    person("Rafael H. M.", "Pereira", , "rafa.pereira.br@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-2125-7465")),
    person("Daniel", "Herszenhut", , "dhersz@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0001-8066-1105")),
    person("Alex", "Magnus", , "alex.magnus@mail.utoronto.ca", role = "aut"),
    person("Matthew Wigginton", "Bhagat-Conway", , "mwbc@unc.edu", role = "aut",
           comment = c(ORCID = "0000-0002-1210-2982")),
    person("Carlos Kaue Vieira", "Braga", , "kaue@kauebraga.dev", role = "ctb",
           comment = c(ORCID = "0000-0002-6104-7297")),
    person("Luyu", "Liu", , "luyuliu@ufl.edu", role = "ctb",
           comment = c(ORCID = "0000-0002-6684-5570")),
    person("Egor", "Kotov", , "kotov.egor@gmail.com", role = c("ctb"),
           comment = c(ORCID = "0000-0001-6690-5345")),
    person("Daniel", "Snow", , "dan@sno.ws", role = "ctb"),
    person("Ipea - Institute for Applied Economic Research", role = c("cph", "fnd")),
    person("Department of Geography & Planning, University of Toronto", role ="fnd")
    )
Description: Rapid realistic routing on multimodal transport networks
    (walk, bike, public transport and car) using 'R5', the Rapid Realistic
    Routing on Real-world and Reimagined networks engine
    <https://github.com/conveyal/r5>. The package allows users to generate
    detailed routing analysis or calculate travel time and monetary cost matrices 
    using seamless parallel computing on top of the R5 Java machine.  While R5
    is developed by Conveyal, the package r5r is independently developed
    by a team at the Institute for Applied Economic Research (Ipea) with
    contributions from collaborators. Apart from the documentation in this
    package, users will find additional information on R5 documentation at
    <https://docs.conveyal.com/>. Although we try to keep new releases of
    r5r in synchrony with R5, the development of R5 follows Conveyal's
    independent update process. Hence, users should confirm the R5 version
    implied by the Conveyal user manual (see
    <https://docs.conveyal.com/changelog>) corresponds with the R5 version
    that r5r depends on. This version of r5r depends on R5 v7.1.
License: MIT + file LICENSE
URL: https://github.com/ipeaGIT/r5r, https://ipeagit.github.io/r5r/
BugReports: https://github.com/ipeaGIT/r5r/issues
Depends: R (>= 3.6)
Imports: checkmate, cli, data.table, isoband, jsonlite, lifecycle,
        methods, purrr, rJava (>= 0.9-10), rlang, sf (>= 1.0-12),
        sfheaders, utils, zip
Suggests: accessibility, covr, dplyr, fs, ggplot2 (>= 3.3.1),
        gtfstools, h3jsr (>= 1.3.0), interp, knitr, mapview, parallel,
        patchwork, rJavaEnv, rmarkdown, testthat
VignetteBuilder: knitr
Encoding: UTF-8
SystemRequirements: Java JDK (>= 21.0)
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-19 20:33:44 UTC; rafap
Author: Marcus Saraiva [aut] (ORCID: <https://orcid.org/0000-0001-6218-2338>),
  Rafael H. M. Pereira [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2125-7465>),
  Daniel Herszenhut [aut] (ORCID:
    <https://orcid.org/0000-0001-8066-1105>),
  Alex Magnus [aut],
  Matthew Wigginton Bhagat-Conway [aut] (ORCID:
    <https://orcid.org/0000-0002-1210-2982>),
  Carlos Kaue Vieira Braga [ctb] (ORCID:
    <https://orcid.org/0000-0002-6104-7297>),
  Luyu Liu [ctb] (ORCID: <https://orcid.org/0000-0002-6684-5570>),
  Egor Kotov [ctb] (ORCID: <https://orcid.org/0000-0001-6690-5345>),
  Daniel Snow [ctb],
  Ipea - Institute for Applied Economic Research [cph, fnd],
  Department of Geography & Planning, University of Toronto [fnd]
Maintainer: Rafael H. M. Pereira <rafa.pereira.br@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-20 06:20:25 UTC
Built: R 4.5.2; ; 2026-05-21 23:32:14 UTC; unix
