#' @section Elevation:
#'
#' More information about the `TOBLER` and `MINETTI` options to calculate the
#' effects of elevation on travel times can be found in the references below:
#'
#' - Campbell, M. J., et al (2019). Using crowdsourced fitness tracker data to
#' model the relationship between slope and travel rates. Applied geography, 106,
#' 93-107. \doi{10.1016/j.apgeog.2019.03.008}.
#' - Minetti, A. E., et al (2002). Energy cost of walking and running at extreme
#' uphill and downhill slopes. Journal of applied physiology. \doi{10.1152/japplphysiol.01177.2001}.
#' - Tobler, W. (1993). Three presentations on geographical analysis and modeling:
#' Non-isotropic geographic modeling speculations on the geometry of geography
#' global spatial analysis. Technical Report. National center for geographic
#' information and analysis. 93 (1). \url{https://escholarship.org/uc/item/05r820mz}.
