#' @section Transport modes:
#'
#' `R5` allows for multiple combinations of transport modes. The options
#' include:
#'
#' - **Transit modes:** `TRAM`, `SUBWAY`, `RAIL`, `BUS`, `FERRY`, `CABLE_CAR`,
#' `GONDOLA`, `FUNICULAR`. The option `TRANSIT` automatically considers all
#' public transport modes available.
#'
#' - **Non transit modes:** `WALK`, `BICYCLE`, `CAR`, `BICYCLE_RENT`,
#' `CAR_PARK`.
