#' @param verbose A logical. Whether to show `R5` informative messages when
#'   running the function. Defaults to `FALSE` (please note that in such case
#'   `R5` error messages are still shown). Setting `verbose` to `TRUE` shows
#'   detailed output, which can be useful for debugging issues not caught by
#'   `r5r`.
