#' @param fare_structure A fare structure object, following the convention
#'   set in [setup_fare_structure()]. This object describes how transit fares
#'   should be calculated. Please see the fare structure vignette to
#'   understand how this object is structured:
#'   `vignette("fare_structure", package = "r5r")`.
