% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/r5r_network.R
\docType{class}
\name{r5r_network-class}
\alias{r5r_network-class}
\title{Class to internally handle Java reference to R5RCore}
\description{
Class to internally handle Java reference to R5RCore
}
\seealso{
Other r5r_network: 
\code{\link{wrap_r5r_network}()}
}
\concept{r5r_network}
\keyword{internal}
