% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expand_od_pairs.R
\name{expand_od_pairs}
\alias{expand_od_pairs}
\title{Expand origin-destination pairs}
\usage{
expand_od_pairs(origins, destinations, all_to_all)
}
\arguments{
\item{origins}{Passed by \code{\link[=detailed_itineraries]{detailed_itineraries()}}.}

\item{destinations}{Passed by \code{\link[=detailed_itineraries]{detailed_itineraries()}}.}

\item{all_to_all}{Passed by \code{\link[=detailed_itineraries]{detailed_itineraries()}}.}
}
\description{
This function is used in \code{\link[=detailed_itineraries]{detailed_itineraries()}} to update the \code{origins} and
\code{destinations} datasets.
}
\keyword{internal}
