% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/java_utils.R
\name{dt_to_lts_map}
\alias{dt_to_lts_map}
\title{data.table to ltsMap}
\usage{
dt_to_lts_map(dt)
}
\arguments{
\item{dt}{data.frame/data.table. Table specifying the
LTS levels. The table must contain columns \code{osm_id} and
\code{lts}.}
}
\value{
A speedMap (Java HashMap<Long, Integer>)
}
\description{
Converts a \code{data.frame} with road OSM id's and respective LTS
levels a Java Map<Long, Integer> for use by r5r_network.
}
\seealso{
Other java support functions: 
\code{\link{dt_to_speed_map}()},
\code{\link{get_java_version}()},
\code{\link{java_to_dt}()}
}
\concept{java support functions}
\keyword{internal}
