% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{exists_tiff}
\alias{exists_tiff}
\title{Check if there is an elevation data file in \code{.tif} format in a given data path}
\usage{
exists_tiff(data_path)
}
\arguments{
\item{data_path}{A string pointing to a directory.}
}
\value{
\code{TRUE} if there is a \code{.tif} file, \code{FALSE} otherwise
}
\description{
Check if there is an elevation data file in \code{.tif} format in a given data path
}
\seealso{
Other support functions: 
\code{\link{fileurl_from_metadata}()},
\code{\link{get_gtfs_errors}()},
\code{\link{start_r5r_java}()},
\code{\link{stop_r5}()},
\code{\link{tempdir_unique}()},
\code{\link{travel_time_surface}},
\code{\link{validate_bad_osm_ids}()}
}
\concept{support functions}
\keyword{internal}
