% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_gtfs_errors.R
\name{get_gtfs_errors}
\alias{get_gtfs_errors}
\title{Get GTFS eventual errors encountered in network building}
\usage{
get_gtfs_errors(r5r_network)
}
\arguments{
\item{r5r_network}{the R5R network object, or a path to the location where
the network is stored (useful if network build failed).}
}
\value{
A \code{data.frame}
}
\description{
This returns a data frame of GTFS errors R5 encountered when building the
network. You can call this with the network itself as the main parameter. If
network build fails, you won't have a network object, so you can also call
this with the \code{data_path} to where the network is stored.
}
\examples{
\dontshow{if (identical(tolower(Sys.getenv("NOT_CRAN")), "true")) withAutoprint(\{ # examplesIf}
library(r5r)

# directory with street network and gtfs files
data_path <- system.file("extdata/poa", package = "r5r")
r5r_network <- build_network(data_path)

get_gtfs_errors(r5r_network)
\dontshow{\}) # examplesIf}
}
\seealso{
Other support functions: 
\code{\link{exists_tiff}()},
\code{\link{fileurl_from_metadata}()},
\code{\link{start_r5r_java}()},
\code{\link{stop_r5}()},
\code{\link{tempdir_unique}()},
\code{\link{travel_time_surface}},
\code{\link{validate_bad_osm_ids}()}
}
\concept{support functions}
