% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assign.R
\name{assign_points_input}
\alias{assign_points_input}
\title{Check and convert origin and destination inputs}
\usage{
assign_points_input(df, name)
}
\arguments{
\item{df}{Either a \code{data.frame} or a \verb{POINT sf}.}

\item{name}{Object name.}
}
\value{
A \code{data.frame} with columns \code{id}, \code{lon} and \code{lat}.
}
\description{
Check and convert origin and destination inputs
}
\seealso{
Other assigning functions: 
\code{\link{assign_decay_function}()},
\code{\link{assign_departure}()},
\code{\link{assign_drop_geometry}()},
\code{\link{assign_max_street_time}()},
\code{\link{assign_max_trip_duration}()},
\code{\link{assign_mode}()},
\code{\link{assign_opportunities}()},
\code{\link{assign_osm_link_ids}()},
\code{\link{assign_shortest_path}()}
}
\concept{assigning functions}
\keyword{internal}
