% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/r5r-package.R
\docType{package}
\name{r5r}
\alias{r5r}
\alias{r5r-package}
\title{r5r: Rapid Realistic Routing with 'R5'}
\description{
Rapid realistic routing on multimodal transport networks (walk, bike, public
transport and car) using \code{R5}, the Rapid Realistic Routing on Real-world and
Reimagined networks engine \url{https://github.com/conveyal/r5}. The package
allows users to generate detailed routing analysis or calculate travel time
matrices using seamless parallel computing on top of the R5 Java machine.
While \code{R5} is developed by Conveyal, the package \code{r5r} is independently
developed by a team at the Institute for Applied Economic Research (Ipea)
with contributions from collaborators. Apart from the documentation in this
package, users will find additional information on \code{R5} documentation at
\url{https://docs.conveyal.com/}. Although we try to keep new releases of \code{r5r}
in synchrony with \code{R5}, the development of \code{R5} follows Conveyal's
independent update process. Hence, users should confirm if the \code{R5} version
implied by the Conveyal user manual (see
\url{https://docs.conveyal.com/changelog}) corresponds with the \code{R5} version
that \code{r5r} depends on.
}
\section{Usage}{

Please check the vignettes on the \href{https://ipeagit.github.io/r5r/}{website}.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ipeaGIT/r5r}
  \item \url{https://ipeagit.github.io/r5r/}
  \item Report bugs at \url{https://github.com/ipeaGIT/r5r/issues}
}

}
\author{
\strong{Maintainer}: Rafael H. M. Pereira \email{rafa.pereira.br@gmail.com} (\href{https://orcid.org/0000-0003-2125-7465}{ORCID})

Authors:
\itemize{
  \item Marcus Saraiva \email{marcus.saraiva@gmail.com} (\href{https://orcid.org/0000-0001-6218-2338}{ORCID})
  \item Daniel Herszenhut \email{dhersz@gmail.com} (\href{https://orcid.org/0000-0001-8066-1105}{ORCID})
  \item Alex Magnus \email{alex.magnus@mail.utoronto.ca}
  \item Matthew Wigginton Bhagat-Conway \email{mwbc@unc.edu} (\href{https://orcid.org/0000-0002-1210-2982}{ORCID})
}

Other contributors:
\itemize{
  \item Carlos Kaue Vieira Braga \email{kaue@kauebraga.dev} (\href{https://orcid.org/0000-0002-6104-7297}{ORCID}) [contributor]
  \item Luyu Liu \email{luyuliu@ufl.edu} (\href{https://orcid.org/0000-0002-6684-5570}{ORCID}) [contributor]
  \item Egor Kotov \email{kotov.egor@gmail.com} (\href{https://orcid.org/0000-0001-6690-5345}{ORCID}) [contributor]
  \item Daniel Snow \email{dan@sno.ws} [contributor]
  \item Ipea - Institute for Applied Economic Research [copyright holder, funder]
  \item Department of Geography & Planning, University of Toronto [funder]
}

}
\keyword{internal}
