% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assign.R
\name{assign_opportunities}
\alias{assign_opportunities}
\title{Assign opportunities data}
\usage{
assign_opportunities(destinations, opportunities_colnames)
}
\arguments{
\item{destinations}{Either a \code{data.frame} or a \verb{POINT sf}.}

\item{opportunities_colnames}{A character vector with the names of the
opportunities columns in \code{destinations}.}
}
\value{
A list of \code{Java-Array} objects.
}
\description{
Check and create an opportunities dataset.
}
\seealso{
Other assigning functions: 
\code{\link{assign_decay_function}()},
\code{\link{assign_departure}()},
\code{\link{assign_drop_geometry}()},
\code{\link{assign_max_street_time}()},
\code{\link{assign_max_trip_duration}()},
\code{\link{assign_mode}()},
\code{\link{assign_osm_link_ids}()},
\code{\link{assign_points_input}()},
\code{\link{assign_shortest_path}()}
}
\concept{assigning functions}
\keyword{internal}
