% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/java_utils.R
\name{java_to_dt}
\alias{java_to_dt}
\title{Java object to data.table}
\usage{
java_to_dt(obj)
}
\arguments{
\item{obj}{A Java Object reference}
}
\value{
An R data.table
}
\description{
Converts a Java object returned by r5r_network to an R \code{data.table}
}
\seealso{
Other java support functions: 
\code{\link{dt_to_lts_map}()},
\code{\link{dt_to_speed_map}()},
\code{\link{get_java_version}()}
}
\concept{java support functions}
\keyword{internal}
