% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stop_r5.R
\name{stop_r5}
\alias{stop_r5}
\title{Stop running r5r network}
\usage{
stop_r5(...)
}
\arguments{
\item{...}{\code{r5r_network} objects currently running. By default, if no
r5r network is supplied all running networks are stopped.}
}
\value{
No return value, called for side effects.
}
\description{
Stops running r5r network
}
\examples{
\dontshow{if (identical(tolower(Sys.getenv("NOT_CRAN")), "true")) withAutoprint(\{ # examplesIf}
library(r5r)

path <- system.file("extdata/poa", package = "r5r")

r5r_network <- build_network(path)

stop_r5(r5r_network)
\dontshow{\}) # examplesIf}
}
\seealso{
Other support functions: 
\code{\link{exists_tiff}()},
\code{\link{fileurl_from_metadata}()},
\code{\link{get_gtfs_errors}()},
\code{\link{start_r5r_java}()},
\code{\link{tempdir_unique}()},
\code{\link{travel_time_surface}},
\code{\link{validate_bad_osm_ids}()}
}
\concept{support functions}
