% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assign.R
\name{assign_departure}
\alias{assign_departure}
\title{Check and convert POSIXct objects to strings}
\usage{
assign_departure(datetime)
}
\arguments{
\item{datetime}{An object of POSIXct class.}
}
\value{
A list with the \code{date} and \code{time} of the trip departure as
characters.
}
\description{
Check and convert POSIXct objects to strings
}
\seealso{
Other assigning functions: 
\code{\link{assign_decay_function}()},
\code{\link{assign_drop_geometry}()},
\code{\link{assign_max_street_time}()},
\code{\link{assign_max_trip_duration}()},
\code{\link{assign_mode}()},
\code{\link{assign_opportunities}()},
\code{\link{assign_osm_link_ids}()},
\code{\link{assign_points_input}()},
\code{\link{assign_shortest_path}()}
}
\concept{assigning functions}
\keyword{internal}
