% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assign.R
\name{assign_mode}
\alias{assign_mode}
\title{Check and select transport modes from user input}
\usage{
assign_mode(mode, mode_egress, style)
}
\arguments{
\item{mode}{A character vector, passed from routing functions.}

\item{mode_egress}{A character vector, passed from routing functions.}
}
\value{
A list with the transport modes to be used in the routing.
}
\description{
Selects the transport modes used in the routing functions. Only one direct
and access/egress modes are allowed at a time.
}
\seealso{
Other assigning functions: 
\code{\link{assign_decay_function}()},
\code{\link{assign_departure}()},
\code{\link{assign_drop_geometry}()},
\code{\link{assign_max_street_time}()},
\code{\link{assign_max_trip_duration}()},
\code{\link{assign_opportunities}()},
\code{\link{assign_osm_link_ids}()},
\code{\link{assign_points_input}()},
\code{\link{assign_shortest_path}()}
}
\concept{assigning functions}
\keyword{internal}
