% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/java_utils.R
\name{get_java_version}
\alias{get_java_version}
\title{Determine the Java version installed locally}
\usage{
get_java_version()
}
\value{
The number of the Java version
}
\description{
Determine the Java version installed locally
}
\seealso{
Other java support functions: 
\code{\link{dt_to_lts_map}()},
\code{\link{dt_to_speed_map}()},
\code{\link{java_to_dt}()}
}
\concept{java support functions}
\keyword{internal}
