% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{tempdir_unique}
\alias{tempdir_unique}
\title{Return a temporary directory path that is unique with every call}
\usage{
tempdir_unique()
}
\value{
Path. Returns the path of the created temporary directory
}
\description{
This is different from the built in tempdir() in that it does not return the same directory within a given runtime. Always returns a unique directory
}
\seealso{
Other support functions: 
\code{\link{exists_tiff}()},
\code{\link{fileurl_from_metadata}()},
\code{\link{get_gtfs_errors}()},
\code{\link{start_r5r_java}()},
\code{\link{stop_r5}()},
\code{\link{travel_time_surface}},
\code{\link{validate_bad_osm_ids}()}
}
\concept{support functions}
\keyword{internal}
