% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{fileurl_from_metadata}
\alias{fileurl_from_metadata}
\title{Get most recent JAR file url from metadata}
\usage{
fileurl_from_metadata(version = NULL)
}
\arguments{
\item{version}{A string. The version of R5 to be downloaded. When \code{NULL}, it
defaults to the latest version.}
}
\value{
A url a string.
}
\description{
Returns the most recent JAR file url from metadata, depending on the version.
}
\seealso{
Other support functions: 
\code{\link{exists_tiff}()},
\code{\link{get_gtfs_errors}()},
\code{\link{start_r5r_java}()},
\code{\link{stop_r5}()},
\code{\link{tempdir_unique}()},
\code{\link{travel_time_surface}},
\code{\link{validate_bad_osm_ids}()}
}
\concept{support functions}
\keyword{internal}
