% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/r5r_network.R
\name{wrap_r5r_network}
\alias{wrap_r5r_network}
\title{Constructor for r5r_network object}
\usage{
wrap_r5r_network(jcore)
}
\arguments{
\item{jcore}{A \code{jobjRef} Java object reference to R5RCore.}
}
\value{
\code{r5r_network}
}
\description{
Wraps a Java R5RCore as an r5r_network.
}
\seealso{
Other r5r_network: 
\code{\link{r5r_network-class}}
}
\concept{r5r_network}
\keyword{internal}
