% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fare_structure.R
\name{assert_fare_structure}
\alias{assert_fare_structure}
\title{Assert fare structure}
\usage{
assert_fare_structure(fare_structure)
}
\arguments{
\item{fare_structure}{A fare structure object, following the convention
set in \code{\link[=setup_fare_structure]{setup_fare_structure()}}. This object describes how transit fares
should be calculated. Please see the fare structure vignette to
understand how this object is structured:
\code{vignette("fare_structure", package = "r5r")}.}
}
\value{
Throws and error upon failure and invisibly returns \code{TRUE} on
success.
}
\description{
Asserts whether the specified fare structure object complies with the
structure set in \code{\link[=setup_fare_structure]{setup_fare_structure()}}.
}
\keyword{internal}
