% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assign.R
\name{assign_decay_function}
\alias{assign_decay_function}
\title{Assign decay function and parameter values}
\usage{
assign_decay_function(decay_function, decay_value)
}
\arguments{
\item{decay_function}{A string, the name of the decay function.}

\item{decay_value}{A number, the value of decay parameter.}
}
\value{
A \code{list} with the validated decay function and parameter value.
}
\description{
Checks and assigns decay function and values.
}
\seealso{
Other assigning functions: 
\code{\link{assign_departure}()},
\code{\link{assign_drop_geometry}()},
\code{\link{assign_max_street_time}()},
\code{\link{assign_max_trip_duration}()},
\code{\link{assign_mode}()},
\code{\link{assign_opportunities}()},
\code{\link{assign_osm_link_ids}()},
\code{\link{assign_points_input}()},
\code{\link{assign_shortest_path}()}
}
\concept{assigning functions}
\keyword{internal}
